using System;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;

namespace ClientControls
{
	/// <summary>
	/// Summary description for UnitSync.
	/// </summary>
	public class UnitSync
	{
		private class UnitSyncFunctions
		{
			[DllImport("unitsync.dll")]
			public static extern Int32 Init(bool isServer, Int32 id);
			[DllImport("unitsync.dll")]
			public static extern void UnInit();
			[DllImport("unitsync.dll")]
			public static extern Int32 ProcessUnits();
			[DllImport("unitsync.dll")]
			public static extern IntPtr GetCurrentList();
			[DllImport("unitsync.dll")]
			public static extern void AddClient(Int32 id, string unitlist);
			[DllImport("unitsync.dll")]
			public static extern void RemoveClient(Int32 id);
			[DllImport("unitsync.dll")]
			public static extern IntPtr GetClientDiff(Int32 id);
			[DllImport("unitsync.dll")]
			public static extern void InstallClientDiff(string diff);
			[DllImport("unitsync.dll")]
			public static extern Int32 GetUnitCount();
			[DllImport("unitsync.dll")]
			public static extern IntPtr GetUnitName(Int32 unit);
			[DllImport("unitsync.dll")]
			public static extern IntPtr GetFullUnitName(Int32 unit);
			[DllImport("unitsync.dll")]
			public static extern Int32 IsUnitDisabled(Int32 unit);
			[DllImport("unitsync.dll")]
			public static extern Int32 IsUnitDisabledByClient(Int32 unit, Int32 clientId);
		}

		private static Boolean		m_bInitialized = false;
		private static Boolean		m_bUnitsProcessed = false;
//		private static StreamWriter		m_SW = null;
		public UnitSync()
		{
			//
			// TODO: Add constructor logic here
			//
			throw new Exception("UnitSync class is a static class, cannot create an instance");
		}

		public static bool Init(bool isServer, int id)
		{
			try
			{
				if ( !m_bInitialized )
				{
					if ( UnitSyncFunctions.Init(isServer, id) != 1 )
					{
						MessageBox.Show("Failed to initialize unit syncer");
						return false;
					}
					else
					{
						ThreadStart ts = new ThreadStart(ProcessUnitsCallback);
						Thread newthread = new Thread(ts);
						newthread.Start();
						m_bInitialized = true;
					}
//					m_SW = new StreamWriter("unitsynclog.txt", false);
//					m_SW.AutoFlush = true;
//					m_SW.WriteLine("Called Init successfully");
					return true;
				}
				return true;
			}
			catch ( ExternalException )
			{
				return false;
			}
			catch ( NullReferenceException )
			{
				m_bInitialized = false;
				return false;
			}
		}

		private static void ProcessUnitsCallback()
		{
			try
			{
				while ( UnitSyncFunctions.ProcessUnits() != 0 );
				lock ( typeof(UnitSync) )
				{
					m_bUnitsProcessed = true;
				}
	/*			
				lock ( m_SW )
				{
					string res = GetCurrentList();
//					m_SW.WriteLine("Current list after processing - {0}", res);
				}*/
			}
			catch ( ExternalException )
			{
			}
		}

		public static int GetUnitCount()
		{
			try
			{
				if ( !m_bInitialized )
					return -1;
				return UnitSyncFunctions.GetUnitCount();
			}
			catch ( ExternalException )
			{
				return -1;
			}
		}

		public static string GetUnitName(int id)
		{
			try
			{
				if ( !m_bInitialized )
					return null;
				IntPtr res = UnitSyncFunctions.GetUnitName(id);
				string ret = Marshal.PtrToStringAnsi(res);
				return ret;
			}
			catch ( ExternalException )
			{
				return null;
			}
			catch ( ArgumentNullException )
			{
				return null;
			}
		}

		public static string GetFullUnitName(int id)
		{
			try
			{
				if ( !m_bInitialized )
					return null;
				IntPtr res = UnitSyncFunctions.GetFullUnitName(id);
				string ret = Marshal.PtrToStringAnsi(res);
				return ret;
			}
			catch ( ExternalException )
			{
				return null;
			}
			catch ( ArgumentNullException )
			{
				return null;
			}
		}

		public static bool IsUnitDisabled(int id)
		{
			try
			{
				if ( !m_bInitialized )
					return false;
				int res = UnitSyncFunctions.IsUnitDisabled(id);
//				m_SW.WriteLine("Called IsUnitDisabled: unit {0} - returned {1}", id, res);
				if ( res == 0 )
					return false;
				return true;
			}
			catch ( ExternalException )
			{
				return false;
			}
		}

		public static bool IsUnitDisabledByClient(int id, int client)
		{
			try
			{
				if ( !m_bInitialized )
					return false;
				int res = UnitSyncFunctions.IsUnitDisabledByClient(id, client);
//				m_SW.WriteLine("Called IsUnitDisabledByClient: client {0}, unit {1} - returned {2}", client, id, res);
				if ( res == 0 )
					return false;
				return true;
			}
			catch ( ExternalException )
			{
				return false;
			}
		}

		public static bool UnitsProcessed()
		{
			bool ret;
			lock ( typeof(UnitSync) )
			{
				ret = m_bUnitsProcessed;
//				m_SW.WriteLine("Called UnitsProcessed: returned {0}", ret);
			}
			return ret;
		}

		public static void Free()
		{
			try
			{
				if ( m_bInitialized )
				{
					UnitSyncFunctions.UnInit();
					m_bInitialized = false;
					m_bUnitsProcessed = false;
//					m_SW.Close();
//					m_SW = null;
				}
			}
			catch ( ExternalException )
			{
			}
		}

		public static bool AddClient(int id, string list)
		{
			try
			{
				if ( m_bInitialized )
				{
					UnitSyncFunctions.AddClient(id, list);
//					m_SW.WriteLine("Called AddClient: with id = {0} and list = {1}", id.ToString(), list);
//					m_SW.WriteLine("List after AddClient - {0}", GetCurrentList());
					return true;
				}
				else
					return false;
			}
			catch ( ExternalException )
			{
				return false;
			}
		}

		public static bool RemoveClient(int id)
		{
			try
			{
				if ( m_bInitialized )
				{
					UnitSyncFunctions.RemoveClient(id);
//					m_SW.WriteLine("Called RemoveClient: with id = {0}", id.ToString());
//					m_SW.WriteLine("List after RemoveClient - {0}", GetCurrentList());
					return true;
				}
				else
					return false;
			}
			catch ( ExternalException )
			{
				return false;
			}
		}

		public static bool InstallClientDiff(string list)
		{
			try
			{
				if ( m_bInitialized )
				{
					UnitSyncFunctions.InstallClientDiff(list);
//					m_SW.WriteLine("Called InstallClientDiff: with list = {0}", list);
//					m_SW.WriteLine("After Call: CurrentList = {0}", GetCurrentList());
					return true;
				}
				return false;
			}
			catch ( ExternalException )
			{
				return false;
			}
		}

		public static string GetClientDiff( int id )
		{
			try
			{
				if ( m_bInitialized )
				{
					IntPtr res = UnitSyncFunctions.GetClientDiff(id);
					string ret = Marshal.PtrToStringAnsi(res);
//					m_SW.WriteLine("Called GetClientDiff: ID = {0}, results = {1}", id.ToString(), ret);
					return ret;
				}
				return null;
			}
			catch ( ExternalException )
			{
				return null;
			}
			catch ( ArgumentNullException )
			{
				return null;
			}
		}

		public static string GetCurrentList()
		{
			try
			{
				if ( m_bInitialized )
				{
					IntPtr res = UnitSyncFunctions.GetCurrentList();
					string ret = Marshal.PtrToStringAnsi(res);
					return ret;
				}
				return null;
			}
			catch ( ExternalException )
			{
				return null;
			}
			catch ( ArgumentNullException )
			{
				return null;
			}
		}

		/*
		static public void Test()
		{
			if ( UnitSyncFunctions.Init(true, 0) != 1 )
			{
				MessageBox.Show("Failed to initialize unit syncer");
				return;
			}

			while ( UnitSyncFunctions.ProcessUnits() != 0 );

			IntPtr str = UnitSyncFunctions.GetCurrentList();
			string list = Marshal.PtrToStringAnsi(str, 100000);
			
			MessageBox.Show(list);

			UnitSyncFunctions.UnInit();
		}*/
	}
}